require 'myxml'
require 'image_cat'

class ConfigReader < MyXMLParser

  attr_writer :config

  def parse(config, filename)
    @config = config
    @image_cats_overridden = false
    super(filename)
  end

  def startElement(name, attrs)
    case name
    when 'option'
      name, value = attrs.indices('name', 'value')
      options = (attrs['inherit'] =~ /^(no|0)$/i) ?
                  @config.non_inherited_options : @config.options
      if name =~ /sort_/ && attrs['order'] && attrs['order'].downcase == 'reverse'
        options[name] = 'reverse ' + value
      else
        options[name] = value
      end
    when 'image'
      image_category = ImageCategory.new(
        attrs['type'], attrs['label'], attrs['longlabel'], attrs['width'], 
        attrs['height']
                                         )
      if @image_cats_overridden
        @config.image_categories.push(image_category)
      else
        # We're overriding the image categories, so start from scratch
        @config.image_categories = [ image_category ]
        @image_cats_overridden = true
      end
    when 'photofield'
      @config.allowed_photofields[attrs['name']] = true
    end
  end

  def endElement(name)
  end

end

# Configuration information class
class Config

  attr_reader :options, :non_inherited_options,:allowed_photofields
              
  attr_accessor :image_categories

  def initialize(parent = DefaultConfig.new)
    @options               = parent.options.dup
    @non_inherited_options = Hash.new
    @image_categories      = parent.image_categories.dup
    @allowed_photofields   = parent.allowed_photofields.dup
  end

  def parse(filename)
    parser = ConfigReader.new
    parser.parse(self, filename)
  end

  def thumbnail_type
    return @thumbnail_type if defined? @thumbnail_type
    tw = get_option('thumb_width')
    th = get_option('thumb_height')
    @thumbnail_type = ImageCategory.new('THUMBNAIL', 'Thumb', 'Thumbnail', tw, th)
    @thumbnail_type
  end
  
  def get_option(opt_name)
    if @non_inherited_options.has_key?(opt_name)
      @non_inherited_options[opt_name]
    else
      @options[opt_name]
    end
  end

  def globbed_ignore_list(dir)
    ignored = Hash.new
    [ global_ignore_list, local_ignore_list(dir) ].each { |list|
      list.each_key { |ignore|
        if ignore =~ /[*?]/
          Dir[dir + '/' + ignore].each { |expanded|
            expanded = expanded.gsub(dir + '/', '')
            ignored[expanded] = 1
          }
        else
          ignored[ignore] = 1
        end
      }
    }
    return ignored
  end

  def global_ignore_list
    return @global_ignore_list if defined? @global_ignore_list

    @global_ignore_list =
      Config.file_to_hash(ENV['HOME'], get_option('ignore_file'))

    return @global_ignore_list
  end

  def local_ignore_list(dir)
    # Not worth caching this.
    return Config.file_to_hash(dir, get_option('ignore_file'))
  end

  def photofield_allowed?(fieldname)
    return @allowed_photofields[fieldname]
  end

  def Config.file_to_hash(dir, file)
    hash = Hash.new
    path = dir + '/' + file
    return hash unless File.exist? path
    File.foreach(path) { |line| hash[line.chomp] = 1 }
    return hash
  end

end

class DefaultConfig < Config

  def initialize
    default_options
    @non_inherited_options = Hash.new
    default_categories
    @allowed_photofields = { 'title' => true, 'order' => true }
  end
  
  # Some sensible defaults for the option values.  Don't change these values manually, 
  # use a config.xml file instead.
  def default_options
    template_dir = ENV['HOME'] + '/.rubyphoto-templates'
    template_dir = '.' unless test(?d, template_dir)

    @options = {
      'template_dir'      => template_dir,
      'album_file'        => 'album.xml',
      'index_file'        => 'index.html',
      'drop_index'        => 'yes',
      'tree_file'         => 'tree.html',
      'ignore_file'       => '.rubyphoto-ignore',
      'table_columns'     => '6',
      'table_rows'        => '4',
      'thumb_width'       => '100',
      'thumb_height'      => '100',
      'sort_photos_by'    => 'filename',
      'sort_subalbums_by' => 'title',
      'next_text'         => 'Next',
      'previous_text'     => 'Prev',
      'min_best_width'    => '321',
      'min_best_height'   => '201',
    }
  end

  def default_categories
    @image_categories = [
      ImageCategory.new('SMALL', 'Sm', 'Small', '320', '200'),
      ImageCategory.new('MEDIUM', 'Med', 'Medium', '640', '480'),
      ImageCategory.new('IMAGE', 'Orig', 'Original')
    ]
  end

end
