require 'image_size'
require 'image_size_cache'
require 'rp_debug'

$html_only = false

class ImageCategory
  
  attr_reader :type_name, :width, :height, :quality

  def initialize(type, label, longlabel,  width=0, height=0, quality=75)
    @type_name = type
    @label     = label
    @longlabel = longlabel
    @width     = width
    @height    = height
    @quality   = quality
  end

  # Probably want better error-handling in this method.
  def create(photo)
    return photo.filename if type_name == "IMAGE"  # represents the original image

    new_filename = image_filename(photo.filename)

    return new_filename unless require_scaling(photo, new_filename)
    
    # Only create a smaller image, never create enlarged versions
    img = ImageSizeCache.open(photo.file_path)

    (got_width, got_height) = img.get_dimensions

    return "" if got_width == nil || got_height == nil
    return new_filename if $html_only && @type_name != 'THUMBNAIL'

    if got_width > width.to_i || got_height > height.to_i
      debug 2, "  Writing #{photo.file_path} at #{geometry} ... ", false
      new_location = photo.directory + "/" + new_filename
      system("convert", "-geometry", geometry,
                        "-quality", quality.to_s,
                        photo.file_path, new_location)
      debug 2, "done."
      return new_filename
    end 

    return ""
  end

  def require_scaling(photo, new_filename)
    existing_scaled = existing_scaled_image(photo, new_filename)

    return true unless existing_scaled

    # If the scaled version exists and is newer than the existing one
    # then don't bother creating it again.
    return File.stat(photo.file_path) > File.stat(existing_scaled)
  end

  def existing_scaled_image(photo, new_filename)
    new_location = photo.directory + '/' + new_filename
    
    # If the original image is a symlink, we can fake up
    # a symlink to the corresponding derived image, and then
    # pretend it already existed.
    if test(?l, photo.file_path)
      create_scaled_symlink(photo, new_filename)
    end

    return File.exists?(new_location) ? new_location : nil
  end

  def create_scaled_symlink(photo, new_filename)
    File.readlink(photo.file_path) =~ %r!(.*/)?(.*)!
    prefix, suffix = $1 || '', $2

    old_dir = Dir.getwd
    Dir.chdir(photo.directory)
    File.unlink(new_filename) if test(?l, new_filename)
    File.symlink(prefix + image_filename($2), new_filename)
    Dir.chdir(old_dir)
  end

  def image_filename(filename)
    return filename if type_name == "IMAGE"
    if filename =~ /^(.+)\.(.*?)$/
      return $1 + "_" + geometry + "." + $2
    else
      raise "weird file: `#{filename}'"
    end
  end

  def page_filename(filename)
    if type_name == "IMAGE"
      return filename + ".html"
    end
    return filename + "." + type_name.downcase + ".html"
  end

  def geometry
    width + "x" + height
  end

  def label(long=false)
    if long
      return @longlabel
    end
    @label
  end

  def dimensions(photo)
    img = ImageSizeCache.open(photo.directory + "/" + image_filename(photo.filename))
    wid, height = img.get_dimensions
    return wid.to_s + " x " + height.to_s
  end

  def filesize(photo)
    byte_size = File.size(photo.directory + "/" + image_filename(photo.filename))
    if byte_size < 1024
      return byte_size.to_s + " bytes"
    end
    return (byte_size/1024).to_s + " KB"
  end

  def image_type(filename)
    filename =~ /^.+\.(.*?)$/
    ext = $1
    if ext =~ /jpg|jpeg/i
      type = "JPEG"
    elsif ext =~ /gif/i
      type = "GIF"
    elsif ext =~ /png/i
      type = "PNG"
    else 
      type = "UNKNOWN"
    end

    return "#{type} image"
  end

end

